import { RequestCookiesAdapter } from '../../server/web/spec-extension/adapters/request-cookies';
import { RequestCookies } from '../../server/web/spec-extension/cookies';
import { staticGenerationAsyncStorage } from '../../client/components/static-generation-async-storage.external';
import { isDynamicIOPrerender, prerenderAsyncStorage } from '../app-render/prerender-async-storage.external';
import { postponeWithTracking, abortAndThrowOnSynchronousDynamicDataAccess, throwToInterruptStaticGeneration, trackDynamicDataInDynamicRender } from '../../server/app-render/dynamic-rendering';
import { getExpectedRequestStore } from '../../client/components/request-async-storage.external';
import { actionAsyncStorage } from '../../client/components/action-async-storage.external';
import { StaticGenBailoutError } from '../../client/components/static-generation-bailout';
import { makeResolvedReactPromise } from './utils';
import { makeHangingPromise } from '../dynamic-rendering-utils';
export function cookies() {
    const callingExpression = 'cookies';
    const requestStore = getExpectedRequestStore(callingExpression);
    const staticGenerationStore = staticGenerationAsyncStorage.getStore();
    const prerenderStore = prerenderAsyncStorage.getStore();
    if (staticGenerationStore) {
        if (staticGenerationStore.forceStatic) {
            // When using forceStatic we override all other logic and always just return an empty
            // cookies object without tracking
            const underlyingCookies = createEmptyCookies();
            return makeUntrackedExoticCookies(underlyingCookies);
        }
        if (staticGenerationStore.isUnstableCacheCallback) {
            throw new Error(`Route ${staticGenerationStore.route} used "cookies" inside a function cached with "unstable_cache(...)". Accessing Dynamic data sources inside a cache scope is not supported. If you need this data inside a cached function use "cookies" outside of the cached function and pass the required dynamic data in as an argument. See more info here: https://nextjs.org/docs/app/api-reference/functions/unstable_cache`);
        } else if (staticGenerationStore.dynamicShouldError) {
            throw new StaticGenBailoutError(`Route ${staticGenerationStore.route} with \`dynamic = "error"\` couldn't be rendered statically because it used \`cookies\`. See more info here: https://nextjs.org/docs/app/building-your-application/rendering/static-and-dynamic#dynamic-rendering`);
        }
        if (prerenderStore) {
            // We are in PPR and/or dynamicIO mode and prerendering
            if (isDynamicIOPrerender(prerenderStore)) {
                // We use the controller and cacheSignal as an indication we are in dynamicIO mode.
                // When resolving cookies for a prerender with dynamic IO we return a forever promise
                // along with property access tracked synchronous cookies.
                // We don't track dynamic access here because access will be tracked when you access
                // one of the properties of the cookies object.
                return makeDynamicallyTrackedExoticCookies(staticGenerationStore.route, prerenderStore);
            } else {
                // We are prerendering with PPR. We need track dynamic access here eagerly
                // to keep continuity with how cookies has worked in PPR without dynamicIO.
                // TODO consider switching the semantic to throw on property access instead
                postponeWithTracking(staticGenerationStore.route, callingExpression, prerenderStore.dynamicTracking);
            }
        } else if (staticGenerationStore.isStaticGeneration) {
            // We are in a legacy static generation mode while prerendering
            // We track dynamic access here so we don't need to wrap the cookies in
            // individual property access tracking.
            throwToInterruptStaticGeneration(callingExpression, staticGenerationStore);
        }
        // We fall through to the dynamic context below but we still track dynamic access
        // because in dev we can still error for things like using cookies inside a cache context
        trackDynamicDataInDynamicRender(staticGenerationStore);
    }
    // cookies is being called in a dynamic context
    const actionStore = actionAsyncStorage.getStore();
    let underlyingCookies;
    // The current implementation of cookies will return Response cookies
    // for a server action during the render phase of a server action.
    // This is not correct b/c the type of cookies during render is ReadOnlyRequestCookies
    // where as the type of cookies during action is ResponseCookies
    // This was found because RequestCookies is iterable and ResponseCookies is not
    if ((actionStore == null ? void 0 : actionStore.isAction) || (actionStore == null ? void 0 : actionStore.isAppRoute)) {
        // We can't conditionally return different types here based on the context.
        // To avoid confusion, we always return the readonly type here.
        underlyingCookies = requestStore.mutableCookies;
    } else {
        underlyingCookies = requestStore.cookies;
    }
    if (process.env.NODE_ENV === 'development' && !(staticGenerationStore == null ? void 0 : staticGenerationStore.isPrefetchRequest)) {
        return makeUntrackedExoticCookiesWithDevWarnings(underlyingCookies, staticGenerationStore == null ? void 0 : staticGenerationStore.route);
    } else {
        return makeUntrackedExoticCookies(underlyingCookies);
    }
}
function createEmptyCookies() {
    return RequestCookiesAdapter.seal(new RequestCookies(new Headers({})));
}
const CachedCookies = new WeakMap();
function makeDynamicallyTrackedExoticCookies(route, prerenderStore) {
    const cachedPromise = CachedCookies.get(prerenderStore);
    if (cachedPromise) {
        return cachedPromise;
    }
    const promise = makeHangingPromise();
    CachedCookies.set(prerenderStore, promise);
    Object.defineProperties(promise, {
        [Symbol.iterator]: {
            value: function() {
                const expression = 'cookies()[Symbol.iterator]()';
                abortAndThrowOnSynchronousDynamicDataAccess(route, expression, prerenderStore);
            }
        },
        size: {
            get () {
                const expression = `cookies().size`;
                abortAndThrowOnSynchronousDynamicDataAccess(route, expression, prerenderStore);
            }
        },
        get: {
            value: function get() {
                let expression;
                if (arguments.length === 0) {
                    expression = 'cookies().get()';
                } else {
                    expression = `cookies().get(${describeNameArg(arguments[0])})`;
                }
                abortAndThrowOnSynchronousDynamicDataAccess(route, expression, prerenderStore);
            }
        },
        getAll: {
            value: function getAll() {
                let expression;
                if (arguments.length === 0) {
                    expression = `cookies().getAll()`;
                } else {
                    expression = `cookies().getAll(${describeNameArg(arguments[0])})`;
                }
                abortAndThrowOnSynchronousDynamicDataAccess(route, expression, prerenderStore);
            }
        },
        has: {
            value: function has() {
                let expression;
                if (arguments.length === 0) {
                    expression = `cookies().has()`;
                } else {
                    expression = `cookies().has(${describeNameArg(arguments[0])})`;
                }
                abortAndThrowOnSynchronousDynamicDataAccess(route, expression, prerenderStore);
            }
        },
        set: {
            value: function set() {
                let expression;
                if (arguments.length === 0) {
                    expression = 'cookies().set()';
                } else {
                    const arg = arguments[0];
                    if (arg) {
                        expression = `cookies().set(${describeNameArg(arg)}, ...)`;
                    } else {
                        expression = `cookies().set(...)`;
                    }
                }
                abortAndThrowOnSynchronousDynamicDataAccess(route, expression, prerenderStore);
            }
        },
        delete: {
            value: function() {
                let expression;
                if (arguments.length === 0) {
                    expression = `cookies().delete()`;
                } else if (arguments.length === 1) {
                    expression = `cookies().delete(${describeNameArg(arguments[0])})`;
                } else {
                    expression = `cookies().delete(${describeNameArg(arguments[0])}, ...)`;
                }
                abortAndThrowOnSynchronousDynamicDataAccess(route, expression, prerenderStore);
            }
        },
        clear: {
            value: function clear() {
                const expression = 'cookies().clear()';
                abortAndThrowOnSynchronousDynamicDataAccess(route, expression, prerenderStore);
            }
        },
        toString: {
            value: function toString() {
                const expression = 'cookies().toString()';
                abortAndThrowOnSynchronousDynamicDataAccess(route, expression, prerenderStore);
            }
        }
    });
    return promise;
}
function makeUntrackedExoticCookies(underlyingCookies) {
    const cachedCookies = CachedCookies.get(underlyingCookies);
    if (cachedCookies) {
        return cachedCookies;
    }
    const promise = makeResolvedReactPromise(underlyingCookies);
    CachedCookies.set(underlyingCookies, promise);
    Object.defineProperties(promise, {
        [Symbol.iterator]: {
            value: underlyingCookies[Symbol.iterator] ? underlyingCookies[Symbol.iterator].bind(underlyingCookies) : // We should remove this and unify our cookies types. We could just let this continue to throw lazily
            // but that's already a hard thing to debug so we may as well implement it consistently. The biggest problem with
            // implementing this in this way is the underlying cookie type is a ResponseCookie and not a RequestCookie and so it
            // has extra properties not available on RequestCookie instances.
            polyfilledResponseCookiesIterator.bind(underlyingCookies)
        },
        size: {
            get () {
                return underlyingCookies.size;
            }
        },
        get: {
            value: underlyingCookies.get.bind(underlyingCookies)
        },
        getAll: {
            value: underlyingCookies.getAll.bind(underlyingCookies)
        },
        has: {
            value: underlyingCookies.has.bind(underlyingCookies)
        },
        set: {
            value: underlyingCookies.set.bind(underlyingCookies)
        },
        delete: {
            value: underlyingCookies.delete.bind(underlyingCookies)
        },
        clear: {
            value: // @ts-expect-error clear is defined in RequestCookies implementation but not in the type
            typeof underlyingCookies.clear === 'function' ? underlyingCookies.clear.bind(underlyingCookies) : // We should remove this and unify our cookies types. We could just let this continue to throw lazily
            // but that's already a hard thing to debug so we may as well implement it consistently. The biggest problem with
            // implementing this in this way is the underlying cookie type is a ResponseCookie and not a RequestCookie and so it
            // has extra properties not available on RequestCookie instances.
            polyfilledResponseCookiesClear.bind(underlyingCookies, promise)
        },
        toString: {
            value: underlyingCookies.toString.bind(underlyingCookies)
        }
    });
    return promise;
}
function makeUntrackedExoticCookiesWithDevWarnings(underlyingCookies, route) {
    const cachedCookies = CachedCookies.get(underlyingCookies);
    if (cachedCookies) {
        return cachedCookies;
    }
    const promise = makeResolvedReactPromise(underlyingCookies);
    CachedCookies.set(underlyingCookies, promise);
    Object.defineProperties(promise, {
        [Symbol.iterator]: {
            value: function() {
                warnForSyncIteration(route);
                return underlyingCookies[Symbol.iterator] ? underlyingCookies[Symbol.iterator].apply(underlyingCookies, arguments) : // We should remove this and unify our cookies types. We could just let this continue to throw lazily
                // but that's already a hard thing to debug so we may as well implement it consistently. The biggest problem with
                // implementing this in this way is the underlying cookie type is a ResponseCookie and not a RequestCookie and so it
                // has extra properties not available on RequestCookie instances.
                polyfilledResponseCookiesIterator.call(underlyingCookies);
            },
            writable: false
        },
        size: {
            get () {
                const expression = 'cookies().size';
                warnForSyncAccess(route, expression);
                return underlyingCookies.size;
            }
        },
        get: {
            value: function get() {
                let expression;
                if (arguments.length === 0) {
                    expression = 'cookies().get()';
                } else {
                    expression = `cookies().get(${describeNameArg(arguments[0])})`;
                }
                warnForSyncAccess(route, expression);
                return underlyingCookies.get.apply(underlyingCookies, arguments);
            },
            writable: false
        },
        getAll: {
            value: function getAll() {
                let expression;
                if (arguments.length === 0) {
                    expression = `cookies().getAll()`;
                } else {
                    expression = `cookies().getAll(${describeNameArg(arguments[0])})`;
                }
                warnForSyncAccess(route, expression);
                return underlyingCookies.getAll.apply(underlyingCookies, arguments);
            },
            writable: false
        },
        has: {
            value: function get() {
                let expression;
                if (arguments.length === 0) {
                    expression = `cookies().has()`;
                } else {
                    expression = `cookies().has(${describeNameArg(arguments[0])})`;
                }
                warnForSyncAccess(route, expression);
                return underlyingCookies.has.apply(underlyingCookies, arguments);
            },
            writable: false
        },
        set: {
            value: function set() {
                let expression;
                if (arguments.length === 0) {
                    expression = 'cookies().set()';
                } else {
                    const arg = arguments[0];
                    if (arg) {
                        expression = `cookies().set(${describeNameArg(arg)}, ...)`;
                    } else {
                        expression = `cookies().set(...)`;
                    }
                }
                warnForSyncAccess(route, expression);
                return underlyingCookies.set.apply(underlyingCookies, arguments);
            },
            writable: false
        },
        delete: {
            value: function() {
                let expression;
                if (arguments.length === 0) {
                    expression = `cookies().delete()`;
                } else if (arguments.length === 1) {
                    expression = `cookies().delete(${describeNameArg(arguments[0])})`;
                } else {
                    expression = `cookies().delete(${describeNameArg(arguments[0])}, ...)`;
                }
                warnForSyncAccess(route, expression);
                return underlyingCookies.delete.apply(underlyingCookies, arguments);
            },
            writable: false
        },
        clear: {
            value: function clear() {
                const expression = 'cookies().clear()';
                warnForSyncAccess(route, expression);
                // @ts-ignore clear is defined in RequestCookies implementation but not in the type
                return typeof underlyingCookies.clear === 'function' ? underlyingCookies.clear.apply(underlyingCookies, arguments) : // We should remove this and unify our cookies types. We could just let this continue to throw lazily
                // but that's already a hard thing to debug so we may as well implement it consistently. The biggest problem with
                // implementing this in this way is the underlying cookie type is a ResponseCookie and not a RequestCookie and so it
                // has extra properties not available on RequestCookie instances.
                polyfilledResponseCookiesClear.call(underlyingCookies, promise);
            },
            writable: false
        },
        toString: {
            value: function toString() {
                const expression = 'cookies().toString()';
                warnForSyncAccess(route, expression);
                return underlyingCookies.toString.apply(underlyingCookies, arguments);
            },
            writable: false
        }
    });
    return promise;
}
function describeNameArg(arg) {
    return typeof arg === 'object' && arg !== null && typeof arg.name === 'string' ? `'${arg.name}'` : typeof arg === 'string' ? `'${arg}'` : '...';
}
function warnForSyncIteration(route) {
    const prefix = route ? ` In route ${route} ` : '';
    console.error(`${prefix}cookies were iterated implicitly with something like \`for...of cookies())\` or \`[...cookies()]\`, or explicitly with \`cookies()[Symbol.iterator]()\`. \`cookies()\` now returns a Promise and the return value should be awaited before attempting to iterate over cookies. In this version of Next.js iterating cookies without awaiting first is still supported to facilitate migration but in a future version you will be required to await the result. If this \`cookies()\` use is inside an async function await the return value before accessing attempting iteration. If this use is inside a synchronous function then convert the function to async or await the call from outside this function and pass the result in.`);
}
function warnForSyncAccess(route, expression) {
    const prefix = route ? ` In route ${route} a ` : 'A ';
    console.error(`${prefix}cookie property was accessed directly with \`${expression}\`. \`cookies()\` now returns a Promise and the return value should be awaited before accessing properties of the underlying cookies instance. In this version of Next.js direct access to \`${expression}\` is still supported to facilitate migration but in a future version you will be required to await the result. If this \`cookies()\` use is inside an async function await the return value before accessing attempting iteration. If this use is inside a synchronous function then convert the function to async or await the call from outside this function and pass the result in.`);
}
function polyfilledResponseCookiesIterator() {
    return this.getAll().map((c)=>[
            c.name,
            c
        ]).values();
}
function polyfilledResponseCookiesClear(returnable) {
    for (const cookie of this.getAll()){
        this.delete(cookie.name);
    }
    return returnable;
}

//# sourceMappingURL=cookies.js.map