"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "headers", {
    enumerable: true,
    get: function() {
        return headers;
    }
});
const _headers = require("../../server/web/spec-extension/adapters/headers");
const _staticgenerationasyncstorageexternal = require("../../client/components/static-generation-async-storage.external");
const _requestasyncstorageexternal = require("../../client/components/request-async-storage.external");
const _prerenderasyncstorageexternal = require("../app-render/prerender-async-storage.external");
const _dynamicrendering = require("../app-render/dynamic-rendering");
const _staticgenerationbailout = require("../../client/components/static-generation-bailout");
const _utils = require("./utils");
const _dynamicrenderingutils = require("../dynamic-rendering-utils");
function headers() {
    const requestStore = (0, _requestasyncstorageexternal.getExpectedRequestStore)('headers');
    const staticGenerationStore = _staticgenerationasyncstorageexternal.staticGenerationAsyncStorage.getStore();
    const prerenderStore = _prerenderasyncstorageexternal.prerenderAsyncStorage.getStore();
    if (staticGenerationStore) {
        if (staticGenerationStore.forceStatic) {
            // When using forceStatic we override all other logic and always just return an empty
            // headers object without tracking
            const underlyingHeaders = _headers.HeadersAdapter.seal(new Headers({}));
            return makeUntrackedExoticHeaders(underlyingHeaders);
        }
        if (staticGenerationStore.isUnstableCacheCallback) {
            throw new Error(`Route ${staticGenerationStore.route} used "headers" inside a function cached with "unstable_cache(...)". Accessing Dynamic data sources inside a cache scope is not supported. If you need this data inside a cached function use "headers" outside of the cached function and pass the required dynamic data in as an argument. See more info here: https://nextjs.org/docs/app/api-reference/functions/unstable_cache`);
        } else if (staticGenerationStore.dynamicShouldError) {
            throw new _staticgenerationbailout.StaticGenBailoutError(`Route ${staticGenerationStore.route} with \`dynamic = "error"\` couldn't be rendered statically because it used \`headers\`. See more info here: https://nextjs.org/docs/app/building-your-application/rendering/static-and-dynamic#dynamic-rendering`);
        }
        if (prerenderStore) {
            // We are in PPR and/or dynamicIO mode and prerendering
            if ((0, _prerenderasyncstorageexternal.isDynamicIOPrerender)(prerenderStore)) {
                // We use the controller and cacheSignal as an indication we are in dynamicIO mode.
                // When resolving headers for a prerender with dynamic IO we return a forever promise
                // along with property access tracked synchronous headers.
                // We don't track dynamic access here because access will be tracked when you access
                // one of the properties of the headers object.
                return makeDynamicallyTrackedExoticHeaders(staticGenerationStore.route, prerenderStore);
            } else {
                // We are prerendering with PPR. We need track dynamic access here eagerly
                // to keep continuity with how headers has worked in PPR without dynamicIO.
                // TODO consider switching the semantic to throw on property access instead
                (0, _dynamicrendering.postponeWithTracking)(staticGenerationStore.route, 'headers', prerenderStore.dynamicTracking);
            }
        } else if (staticGenerationStore.isStaticGeneration) {
            // We are in a legacy static generation mode while prerendering
            // We track dynamic access here so we don't need to wrap the headers in
            // individual property access tracking.
            (0, _dynamicrendering.throwToInterruptStaticGeneration)('headers', staticGenerationStore);
        }
        // We fall through to the dynamic context below but we still track dynamic access
        // because in dev we can still error for things like using headers inside a cache context
        (0, _dynamicrendering.trackDynamicDataInDynamicRender)(staticGenerationStore);
    }
    if (process.env.NODE_ENV === 'development' && !(staticGenerationStore == null ? void 0 : staticGenerationStore.isPrefetchRequest)) {
        return makeUntrackedExoticHeadersWithDevWarnings(requestStore.headers, staticGenerationStore == null ? void 0 : staticGenerationStore.route);
    } else {
        return makeUntrackedExoticHeaders(requestStore.headers);
    }
}
const CachedHeaders = new WeakMap();
function makeDynamicallyTrackedExoticHeaders(route, prerenderStore) {
    const cachedHeaders = CachedHeaders.get(prerenderStore);
    if (cachedHeaders) {
        return cachedHeaders;
    }
    const promise = (0, _dynamicrenderingutils.makeHangingPromise)();
    CachedHeaders.set(prerenderStore, promise);
    Object.defineProperties(promise, {
        append: {
            value: function append() {
                const expression = `headers().append(${describeNameArg(arguments[0])}, ...)`;
                (0, _dynamicrendering.abortAndThrowOnSynchronousDynamicDataAccess)(route, expression, prerenderStore);
            }
        },
        delete: {
            value: function _delete() {
                const expression = `headers().delete(${describeNameArg(arguments[0])})`;
                (0, _dynamicrendering.abortAndThrowOnSynchronousDynamicDataAccess)(route, expression, prerenderStore);
            }
        },
        get: {
            value: function get() {
                const expression = `headers().get(${describeNameArg(arguments[0])})`;
                (0, _dynamicrendering.abortAndThrowOnSynchronousDynamicDataAccess)(route, expression, prerenderStore);
            }
        },
        has: {
            value: function has() {
                const expression = `headers().has(${describeNameArg(arguments[0])})`;
                (0, _dynamicrendering.abortAndThrowOnSynchronousDynamicDataAccess)(route, expression, prerenderStore);
            }
        },
        set: {
            value: function set() {
                const expression = `headers().set(${describeNameArg(arguments[0])}, ...)`;
                (0, _dynamicrendering.abortAndThrowOnSynchronousDynamicDataAccess)(route, expression, prerenderStore);
            }
        },
        getSetCookie: {
            value: function getSetCookie() {
                const expression = `headers().getSetCookie()`;
                (0, _dynamicrendering.abortAndThrowOnSynchronousDynamicDataAccess)(route, expression, prerenderStore);
            }
        },
        forEach: {
            value: function forEach() {
                const expression = `headers().forEach(...)`;
                (0, _dynamicrendering.abortAndThrowOnSynchronousDynamicDataAccess)(route, expression, prerenderStore);
            }
        },
        keys: {
            value: function keys() {
                const expression = `headers().keys()`;
                (0, _dynamicrendering.abortAndThrowOnSynchronousDynamicDataAccess)(route, expression, prerenderStore);
            }
        },
        values: {
            value: function values() {
                const expression = `headers().values()`;
                (0, _dynamicrendering.abortAndThrowOnSynchronousDynamicDataAccess)(route, expression, prerenderStore);
            }
        },
        entries: {
            value: function entries() {
                const expression = `headers().entries()`;
                (0, _dynamicrendering.abortAndThrowOnSynchronousDynamicDataAccess)(route, expression, prerenderStore);
            }
        },
        [Symbol.iterator]: {
            value: function() {
                const expression = 'headers()[Symbol.iterator]()';
                (0, _dynamicrendering.abortAndThrowOnSynchronousDynamicDataAccess)(route, expression, prerenderStore);
            }
        }
    });
    return promise;
}
function makeUntrackedExoticHeaders(underlyingHeaders) {
    const cachedHeaders = CachedHeaders.get(underlyingHeaders);
    if (cachedHeaders) {
        return cachedHeaders;
    }
    const promise = (0, _utils.makeResolvedReactPromise)(underlyingHeaders);
    CachedHeaders.set(underlyingHeaders, promise);
    Object.defineProperties(promise, {
        append: {
            value: underlyingHeaders.append.bind(underlyingHeaders)
        },
        delete: {
            value: underlyingHeaders.delete.bind(underlyingHeaders)
        },
        get: {
            value: underlyingHeaders.get.bind(underlyingHeaders)
        },
        has: {
            value: underlyingHeaders.has.bind(underlyingHeaders)
        },
        set: {
            value: underlyingHeaders.set.bind(underlyingHeaders)
        },
        getSetCookie: {
            value: underlyingHeaders.getSetCookie.bind(underlyingHeaders)
        },
        forEach: {
            value: underlyingHeaders.forEach.bind(underlyingHeaders)
        },
        keys: {
            value: underlyingHeaders.keys.bind(underlyingHeaders)
        },
        values: {
            value: underlyingHeaders.values.bind(underlyingHeaders)
        },
        entries: {
            value: underlyingHeaders.entries.bind(underlyingHeaders)
        },
        [Symbol.iterator]: {
            value: underlyingHeaders[Symbol.iterator].bind(underlyingHeaders)
        }
    });
    return promise;
}
function makeUntrackedExoticHeadersWithDevWarnings(underlyingHeaders, route) {
    const cachedHeaders = CachedHeaders.get(underlyingHeaders);
    if (cachedHeaders) {
        return cachedHeaders;
    }
    const promise = (0, _utils.makeResolvedReactPromise)(underlyingHeaders);
    CachedHeaders.set(underlyingHeaders, promise);
    Object.defineProperties(promise, {
        append: {
            value: function append() {
                const expression = `headers().append(${describeNameArg(arguments[0])}, ...)`;
                warnForSyncAccess(route, expression);
                return underlyingHeaders.append.apply(underlyingHeaders, arguments);
            }
        },
        delete: {
            value: function _delete() {
                const expression = `headers().delete(${describeNameArg(arguments[0])})`;
                warnForSyncAccess(route, expression);
                return underlyingHeaders.delete.apply(underlyingHeaders, arguments);
            }
        },
        get: {
            value: function get() {
                const expression = `headers().get(${describeNameArg(arguments[0])})`;
                warnForSyncAccess(route, expression);
                return underlyingHeaders.get.apply(underlyingHeaders, arguments);
            }
        },
        has: {
            value: function has() {
                const expression = `headers().has(${describeNameArg(arguments[0])})`;
                warnForSyncAccess(route, expression);
                return underlyingHeaders.has.apply(underlyingHeaders, arguments);
            }
        },
        set: {
            value: function set() {
                const expression = `headers().set(${describeNameArg(arguments[0])}, ...)`;
                warnForSyncAccess(route, expression);
                return underlyingHeaders.set.apply(underlyingHeaders, arguments);
            }
        },
        getSetCookie: {
            value: function getSetCookie() {
                const expression = `headers().getSetCookie()`;
                warnForSyncAccess(route, expression);
                return underlyingHeaders.getSetCookie.apply(underlyingHeaders, arguments);
            }
        },
        forEach: {
            value: function forEach() {
                const expression = `headers().forEach(...)`;
                warnForSyncAccess(route, expression);
                return underlyingHeaders.forEach.apply(underlyingHeaders, arguments);
            }
        },
        keys: {
            value: function keys() {
                const expression = `headers().keys()`;
                warnForSyncAccess(route, expression);
                return underlyingHeaders.keys.apply(underlyingHeaders, arguments);
            }
        },
        values: {
            value: function values() {
                const expression = `headers().values()`;
                warnForSyncAccess(route, expression);
                return underlyingHeaders.values.apply(underlyingHeaders, arguments);
            }
        },
        entries: {
            value: function entries() {
                const expression = `headers().entries()`;
                warnForSyncAccess(route, expression);
                return underlyingHeaders.entries.apply(underlyingHeaders, arguments);
            }
        },
        [Symbol.iterator]: {
            value: function() {
                warnForSyncIteration(route);
                return underlyingHeaders[Symbol.iterator].apply(underlyingHeaders, arguments);
            }
        }
    });
    return promise;
}
function describeNameArg(arg) {
    return typeof arg === 'string' ? `'${arg}'` : '...';
}
function warnForSyncIteration(route) {
    const prefix = route ? ` In route ${route} ` : '';
    console.error(`${prefix}headers were iterated implicitly with something like \`for...of headers())\` or \`[...headers()]\`, or explicitly with \`headers()[Symbol.iterator]()\`. \`headers()\` now returns a Promise and the return value should be awaited before attempting to iterate over headers. In this version of Next.js iterating headers without awaiting first is still supported to facilitate migration but in a future version you will be required to await the result. If this \`headers()\` use is inside an async function await the return value before accessing attempting iteration. If this use is inside a synchronous function then convert the function to async or await the call from outside this function and pass the result in.`);
}
function warnForSyncAccess(route, expression) {
    const prefix = route ? ` In route ${route} a ` : 'A ';
    console.error(`${prefix}header property was accessed directly with \`${expression}\`. \`headers()\` now returns a Promise and the return value should be awaited before accessing properties of the underlying headers instance. In this version of Next.js direct access to \`${expression}\` is still supported to facilitate migration but in a future version you will be required to await the result. If this \`headers()\` use is inside an async function await the return value before accessing attempting iteration. If this use is inside a synchronous function then convert the function to async or await the call from outside this function and pass the result in.`);
}

//# sourceMappingURL=headers.js.map